!------------------------------------------------------------------------
! Subroutines that perform various dumps and that can load dump files
!------------------------------------------------------------------------

recursive subroutine Dumppoints(DestDir,Nx,X,Ny,Y,Nz,Z,gl,attemptno,Nretries)

  implicit none

  character(len=*) :: DestDir
  integer Nx, Ny, Nz, gl, attemptno, Nretries
  double precision X(Nx), Y(Ny), Z(Nz)

  character filename*80, glnumber*2
  integer ix, iy, iz, ierr

  attemptno = attemptno + 1

  ! Assemble filename
  write(glnumber,fmt='(i2.2)') gl
  filename = DestDir//'pointfile'//glnumber//'.cheese.dat'

  ! open file and write grid points
  open(unit=1,file=filename,status='replace',err=97)
  do ix = 1, Nx
     do iy = 1, Ny
        do iz = 1, Nz
           write (1,fmt='(E16.9E3,a,E16.9E3,a,E16.9E3)',err=98) &
                X(ix), ' ', Y(iy), ' ', Z(iz)
        end do
     end do
  end do
  close(1,err=99)
  return


! Error handling section
97 write (*,*) 'Dumppoints: error in open statement'
  goto 100
98 write (*,*) 'Dumppoints: error in write statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'Dumppoints: error in close statement'
100 if (attemptno<=Nretries) then
     call Dumppoints(DestDir,Nx,X,Ny,Y,Nz,Z,gl,attemptno,Nretries)
  end if

end subroutine Dumppoints

!------------------------------------------------------------------------

recursive subroutine DumpScalarField(Field,Nx,Ny,Nz,DestDir,fieldname, &
     iteration, attemptno, Nretries)

! No implicit varaibles
  implicit none

! Parameters
  character(len=*) :: DestDir, fieldname
  integer Nx, Ny, Nz, iteration, attemptno, Nretries
  double precision Field(Nx,Ny,Nz)
!!  character fieldname*4

! Local variables
  integer ii, jj, kk
  character filename*200, number*10

  attemptno = attemptno + 1

  write(number,fmt='(i10.10)') iteration

! Filename for scalar field
  filename = DestDir//fieldname//number//'.cheese.dat'


! Create and open file for writing vector field
  open(unit=1,file=filename,status='replace',err=97)

  do kk = 1, Nz
     do jj = 1, Ny
        do ii = 1, Nx
           write (1,fmt='(E12.6)',err=98) Field(ii,jj,kk)
        end do
     end do
  end do

  close(1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpScalarField: error in open statement'
  goto 100
98 write (*,*) 'DumpScalarField: error in write statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpScalarField: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpScalarField(Field,Nx,Ny,Nz,DestDir,fieldname, &
          iteration,attemptno,Nretries)
  end if

end subroutine DumpScalarField

!------------------------------------------------------------------------

recursive subroutine DumpVectorField(Field,Nx,Ny,Nz,DestDir,fieldname, &
     iteration,attemptno,Nretries)


! No implicit varaibles
  implicit none

! Parameters
  character(len=*) :: DestDir, fieldname
  integer Nx, Ny, Nz, iteration, attemptno, Nretries
  double precision Field(3,Nx,Ny,Nz)
!!$  character fieldname*7

! Local variables
  integer ii, jj, kk, ll
  character filename*200, number*10

  attemptno = attemptno + 1

  write(number,fmt='(i10.10)') iteration

! Filename for vector field
  filename = DestDir//fieldname//number//'.cheese.dat'


! Create and open file for writing vector field
  open(unit=1,file=filename,status='replace',err=97)

  do kk = 1, Nz
     do jj = 1, Ny
        do ii = 1, Nx
           write (1,fmt='(E12.6,a,E12.6,a,E12.6)',err=98) &
                Field(1,ii,jj,kk), ' ', Field(2,ii,jj,kk), ' ', &
                Field(3,ii,jj,kk)
        end do
     end do
  end do

  close(1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpVectorField: error in open statement'
  goto 100
98 write (*,*) 'DumpVectorField: error in write statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpVectorField: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpVectorField(Field,Nx,Ny,Nz,DestDir,fieldname, &
          iteration,attemptno,Nretries)
  end if

end subroutine DumpVectorField

!------------------------------------------------------------------------

recursive subroutine DumpFieldDump(B, E, gl, Nx, Ny, Nz, attemptno, Nretries)


! No implicit variables
  implicit none

! Parameters
  integer gl, Nx, Ny, Nz, attemptno, Nretries
  double precision B(3,Nx,Ny,Nz), E(3,Nx,Ny,Nz)

! Local variables
  character filename*60, glnumber*2

  attemptno = attemptno + 1

! Filename for B field
  write(glnumber,fmt='(i2.2)') gl
  filename = 'dumps/dump_Bfield_'//glnumber//'.cheese.bin'

! Create and open file for writing potential
  open(unit=1,form='unformatted',file=filename,status='replace',err=97)
  write (1,err=98) B
  close (1,err=99)


! Filename for E field
  filename = 'dumps/dump_Efield_'//glnumber//'.cheese.bin'

! Create and open file for writing potential
  open(unit=1,form='unformatted',file=filename,status='replace',err=97)
  write (1,err=98) E
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpFieldDump: error in open statement'
  goto 100
98 write (*,*) 'DumpFieldDump: error in write statement'
  close (1,err=99)
  goto 100
99 write (*,*) 'DumpFieldDump: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpFieldDump(B, E, gl, Nx, Ny, Nz, attemptno, Nretries)
  end if


end subroutine DumpFieldDump

!------------------------------------------------------------------------

recursive subroutine LoadFieldDump(B, E, gl, Nx, Ny, Nz, attemptno, Nretries)

! No implicit variables
  implicit none

! Parameters
  integer gl, Nx, Ny, Nz, attemptno, Nretries
  double precision B(3,Nx,Ny,Nz), E(3,Nx,Ny,Nz)

! Local variables
  character filename*60, glnumber*2


! Filename for B field
  write(glnumber,fmt='(i2.2)') gl
  filename = 'dumps/dump_Bfield_'//glnumber//'.cheese.bin'

! Create and open file for reading potential
  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  read (1,err=98) B
  close (1,err=99)

  attemptno = attemptno + 1

! Filename for E field
  filename = 'dumps/dump_Efield_'//glnumber//'.cheese.bin'

! Create and open file for reading E-field
  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  read (1,err=98) E
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'LoadFieldDump: error in open statement'
  goto 100
98 write (*,*) 'LoadFieldDump: error in read statement'
  close (1,err=99)
  goto 100
99 write (*,*) 'LoadFieldDump: error in close statement'
100 if (attemptno<=Nretries) then
     call LoadFieldDump(B, E, gl, Nx, Ny, Nz, attemptno, Nretries)
  end if

end subroutine LoadFieldDump

!------------------------------------------------------------------------

recursive subroutine DumpParticles(Npart, pos, vel, nextno, attemptno, Nretries)

  implicit none

! Arguments
  integer Npart, nextno, attemptno, Nretries
  double precision pos(3,Npart), vel(3,Npart)

! Local variables
  character filename*200

  attemptno = attemptno + 1

! Write nextno
  filename = 'dumps/dump_nextno.cheese.dump'
  open(unit=1,file=filename,status='replace',err=97)
  write (1,fmt='(I10)',err=98) nextno
  close (1,err=99)


! Filename for particles
  filename = 'dumps/dump_particles.cheese.dump'

! Create and open file for writing particle positions, velocities, and nextno
  open(unit=1,form='unformatted',file=filename,status='replace',err=97)
  write (1,err=98) pos
  write (1,err=98) vel
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpParticles: error in open statement'
  goto 100
98 write (*,*) 'DumpParticles: error in write statement'
  close (1,err=99)
  goto 100
99 write (*,*) 'DumpParticles: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpParticles(Npart, pos, vel, nextno, attemptno, Nretries)
  end if

end subroutine DumpParticles

!------------------------------------------------------------------------

recursive subroutine LoadParticles(Npart, pos, vel, nextno, attemptno, Nretries)

  implicit none

! Arguments
  integer Npart, nextno, attemptno, Nretries
  double precision pos(3,Npart), vel(3,Npart)

! Local variables
  character filename*200

  attemptno = attemptno + 1

! Read nextno
  filename = 'dumps/dump_nextno.cheese.dump'
  open(unit=1,file=filename,status='old',err=97)
  read (1,fmt='(I10)',err=98) nextno
  close (1,err=99)

! Filename for particles
  filename = 'dumps/dump_particles.cheese.dump'

! Create and open file for writing particle positions, velocities, and nextno
  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  read (1,err=98) pos
  read (1,err=98) vel
  close(1,err=99)

  return

! Error handling section
97 write (*,*) 'LoadParticles: error in open statement'
  goto 100
98 write (*,*) 'LoadParticles: error in read statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'LoadParticles: error in close statement'
100 if (attemptno<=Nretries) then
     call LoadParticles(Npart, pos, vel, nextno, attemptno, Nretries)
  end if

end subroutine LoadParticles

!------------------------------------------------------------------------

recursive subroutine DumpDensityDump(Nx, Ny, Nz, Field, &
     nextno, attemptno, Nretries)

  implicit none

  integer Nx, Ny, Nz, nextno, attemptno, Nretries
  double precision Field(Nx,Ny,Nz)

! Local variables
  character filename*200

  attemptno = attemptno + 1

! Write nextno
  filename = 'dumps/dump_nextno.cheese.dump'
  open(unit=1,file=filename,status='replace',err=97)
  write (1,fmt='(I10)',err=98) nextno
  close (1,err=99)


! Filename for Field
  filename = 'dumps/dump_dens.cheese.dump'

! Create and open file for writing Field
  open(unit=1,form='unformatted',file=filename,status='replace',err=97)
  write (1,err=98) Field
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpDensityDump: error in open statement'
  goto 100
98 write (*,*) 'DumpdensityDump: error in write statement'
  close (1,err=99)
  goto 100
99 write (*,*) 'DumpDensityDump: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpDensityDump(Nx, Ny, Nz, Field, &
          nextno, attemptno, Nretries)
  end if

end subroutine DumpDensityDump

!------------------------------------------------------------------------

recursive subroutine LoadDensityDump(Nx, Ny, Nz, Field, &
     nextno, attemptno, Nretries)

  implicit none

  integer Nx, Ny, Nz, nextno, attemptno, Nretries
  double precision Field(Nx,Ny,Nz)

! Local variables
  character filename*200

  attemptno = attemptno + 1

! Write nextno
  filename = 'dumps/dump_nextno.cheese.dump'
  open(unit=1,file=filename,status='old',err=97)
  read (1,fmt='(I10)',err=98) nextno
  close (1,err=99)


! Filename for Field
  filename = 'dumps/dump_dens.cheese.dump'

! Create and open file for writing Field
  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  read (1,err=98) Field
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'LoadDensityDump: error in open statement'
  goto 100
98 write (*,*) 'LoaddensityDump: error in read statement'
  close (1,err=99)
  goto 100
99 write (*,*) 'LoadDensityDump: error in close statement'
100 if (attemptno<=Nretries) then
     call LoadDensityDump(Nx, Ny, Nz, Field, &
          nextno, attemptno, Nretries)
  end if

end subroutine LoadDensityDump

!------------------------------------------------------------------------

recursive subroutine DumpProbeDump(Nprobes, probe, attemptno, Nretries)

  use SpecificTypes

  implicit none

  integer Nprobes, attemptno, Nretries
  type(fprobe) probe(Nprobes)

  integer ii
  character filename*200

  attemptno = attemptno + 1

  filename='dumps/dump_probe.cheese.dump' 

  open(unit=1,form='unformatted',file=filename,status='replace',err=97)
  do ii = 1, Nprobes
     write (1,err=98) probe(ii)%f
  end do
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'DumpProbeDump: error in open statement'
  goto 100
98 write (*,*) 'DumpProbeDump: error in write statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'DumpProbeDump: error in close statement'
100 if (attemptno<=Nretries) then
     call DumpProbeDump(Nprobes, probe, attemptno, Nretries)
  end if

end subroutine DumpProbeDump

!------------------------------------------------------------------------

recursive subroutine LoadProbeDump(Nprobes, probe, attemptno, Nretries)

  use SpecificTypes

  implicit none

  integer Nprobes, attemptno, Nretries
  type(fprobe) probe(Nprobes)

  integer ii
  character filename*200

  attemptno = attemptno + 1

  filename='dumps/dump_probe.cheese.dump' 

  open(unit=1,form='unformatted',file=filename,status='old',err=97)
  do ii = 1, Nprobes
     read (1,err=98) probe(ii)%f
  end do
  close (1,err=99)

  return

! Error handling section
97 write (*,*) 'LoadProbeDump: error in open statement'
  goto 100
98 write (*,*) 'LoadProbeDump: error in read statement'
  close(1,err=99)
  goto 100
99 write (*,*) 'LoadProbeDump: error in close statement'
100 if (attemptno<=Nretries) then
     call LoadProbeDump(Nprobes, probe, attemptno, Nretries)
  end if

end subroutine LoadProbeDump

!------------------------------------------------------------------------


