module SpecificTypes

  type species
   ! read from input file
     character hybfilename*200 ! Name of hybrid data file
     double precision chargestate ! Number or elementary charges per ion
     integer hybiteration      ! Iteration of the hybrid simulation
   ! read from hybrid data file
     double precision, allocatable :: B(:,:,:,:), vel(:,:,:,:), &
          cur(:,:,:,:), rho(:,:,:), dens(:,:,:)
  end type species

  type InputGeneral
     ! General input parameters that are read from the input file
     integer NhybSpecies,Npart, Nsteps, Ngridlevels, Nprobes, initialiser
     integer attemptno, Nretries
     integer dump_period_dump
     integer Nx, Ny, Nz
     double precision xmin, xmax, ymin, ymax, zmin, zmax
     double precision Xcube, Ycube, Zcube
     double precision vxbulk, vybulk, vzbulk, vrbulk, vth, Lside
     double precision dt, mass, charge, robst, rsource
     double precision thetamin, thetamax, azmin, azmax
     double precision saving_step_length 
     logical startfromdumpfile
     character scdatafilename*200, magfilename*200, curfilename*200
     ! Flucutation model
     logical Efluctuations, includephotonpressure, usealpha
     double precision rinfluct, routfluct, E1x, E1y, E1z, ffluct, alpha
     ! Dust specific version, DustyCheddar, needs this
     double precision Flight
  end type InputGeneral

  type GridLevel
     ! Parameters defining a grid and the fields that are stored on that grid
     integer Nx, Ny, Nz
     double precision xmin, xmax, ymin, ymax, zmin, zmax, Lx, Ly, Lz, dxyz(3)
     double precision, allocatable :: B(:,:,:,:), E(:,:,:,:)
  end type GridLevel

  type fprobe
   ! Distribution function probe
   ! read from input file
     integer Nvx, Nvy, Nvz
     ! coordinates for the centre of the probe and the probe radius
     double precision xc, yc, zc, rprobe
     ! boundaries of the part of velocity space considered
     double precision vxmin, vxmax, vymin, vymax, vzmin, vzmax
   ! calculated
     ! Vz is the initial velocity vector comprising Nvz points from 
     ! vzmin to vzmax in steps of dvz=(vzmax-vzmin)/(Nvz-1)
     double precision dvx, dvy, dvz
     double precision, allocatable :: Vx(:), Vy(:), Vz(:)
     ! f is a (Nvx, Nvy, Nvz) array, containing the distribution function
     double precision, allocatable :: f(:,:,:)
  end type fprobe

end module SpecificTypes

