! This program computes trajectories and saves them. One line per time step.


program cheese

  use SpecificTypes

  implicit none

  type (InputGeneral) inpar

  integer ii, jj, sjj, kk, gl
  
  double precision B0(3)

  double precision dt, rr, pi

  double precision E0(3), Eperp(3), step_since_saving(3)
  type (GridLevel), allocatable :: grid(:)

  double precision, allocatable :: position_init(:,:), velocity_init(:,:)
	   
  double precision, allocatable :: position(:,:), saving_position(:,:), &
       velocity(:,:), vv(:), mu(:), pangle(:)

  double precision E1(3), E1par(3), E1perp(3), E1mag, rndph, fluctphase

  integer Nretries, attempts


! Initialise a couple of things
  parameter(pi = 3.14159265358979323846264338327950288419716d0)
  parameter(Nretries = 10)
 

! Read input data
  attempts = 0
  call GetGeneralInput(inpar, attempts, Nretries)

  allocate(grid(inpar%Ngridlevels))
  attempts = 0
  call GetGridLevels('', inpar%Ngridlevels, grid, attempts, Nretries)
  do gl = 1, inpar%Ngridlevels
     ! Grid size
     grid(gl)%Lx = grid(gl)%xmax - grid(gl)%xmin
     grid(gl)%Ly = grid(gl)%ymax - grid(gl)%ymin
     grid(gl)%Lz = grid(gl)%zmax - grid(gl)%zmin
     ! Grid cell size
     grid(gl)%dxyz(1) = grid(gl)%Lx/(grid(gl)%Nx-1)
     grid(gl)%dxyz(2) = grid(gl)%Ly/(grid(gl)%Ny-1)
     grid(gl)%dxyz(3) = grid(gl)%Lz/(grid(gl)%Nz-1)
     ! Field allocation
     allocate(grid(gl)%E(3,grid(gl)%Nx,grid(gl)%Ny,grid(gl)%Nz))
     allocate(grid(gl)%B(3,grid(gl)%Nx,grid(gl)%Ny,grid(gl)%Nz))
     ! Field loading
     attempts = 0
     call LoadFieldDump(grid(gl)%B, grid(gl)%E, gl, &
          grid(gl)%Nx, grid(gl)%Ny, grid(gl)%Nz, attempts, Nretries)
  end do


 ! Initialise timestep variable to the value in the input file
  dt           = inpar%dt ! dt



! Allocate variables for initial position and velocity, and for 
! particle position and velocity during flight.
  allocate(position_init(3,inpar%Npart))
  allocate(velocity_init(3,inpar%Npart))
  allocate(position(3,inpar%Nsteps))
  allocate(saving_position(3,inpar%Nsteps))
  allocate(velocity(3,inpar%Nsteps))
  allocate(vv(inpar%Nsteps))
  allocate(mu(inpar%Nsteps))
  allocate(pangle(inpar%Nsteps))


! Give us particles! 
! This is done differently depending on the initialiser that is chosen 
! in the file inputcheese.m.
  select case (inpar%initialiser)
  case(1)
     call InitialiseParticles_01(velocity_init,position_init, &
          inpar%Xcube, inpar%Ycube, inpar%Zcube, inpar%vxbulk, & 
          inpar%vybulk, inpar%vzbulk, inpar%vth, inpar%Lside, inpar%Npart)

  case(2)
     call InitialiseParticles_02(velocity_init, position_init, &
          inpar%Npart, inpar%mass, inpar%scdatafilename, &
          inpar%Ngridlevels, grid)

  case(3)
     call InitialiseParticles_03(velocity_init, position_init, &
          inpar%thetamin, inpar%thetamax, inpar%azmin, inpar%azmax, &
          inpar%vrbulk, inpar%vth, inpar%rsource, inpar%Npart)

  case default
     write (*,*) 'initialiser =', inpar%initialiser, ' is illegal!'
     stop
  end select
  
! Open the output file. Any old file by the same name will 
! be exterminated.
  open(unit=2, file='outp/datfiles/trajectories/trajectories.dat', &
       status='replace')

  
! Here starts the main loop that computes the trajectories and saves them
  do ii=1, inpar%Npart

!trajectory saving index reset for next particle
     sjj = 1
     step_since_saving = 0.0d0
	
! Initial values for the present particle
     position(:,1) = position_init(:,ii)
     velocity(:,1) = velocity_init(:,ii)

! Find initial fields through interpolation
     call FindFields(inpar%Ngridlevels, grid, &
          position(1,1), position(2,1), position(3,1), E0, B0)

! Compute perpendicular E, i.e. remove artificial parallel component
     call PerpendicularEfield(E0, B0, Eperp)

! Find fluctuation E field if we are including such
     if (inpar%Efluctuations) then
        ! To each particle a random phase
        call random_number(rndph)
        fluctphase = 2*pi*rndph
        call FindFluct(inpar%rinfluct, inpar%routfluct, &
             inpar%E1x, inpar%E1y, inpar%E1z, inpar%ffluct,  &
             position(1,1), position(2,1), position(3,1), &
             fluctphase, 0.0d0, E1)
        E1mag = sqrt(sum(E1**2.0d0))
        if ( (inpar%usealpha) .and. (E1mag > 0.0d0) ) then
           call PerpendicularEfield(E1, B0, E1perp)
           E1par = E1 - E1perp
           E1 = E1mag * ( &
                sin(inpar%alpha)*E1par/sqrt(sum(E1par**2.0d0)) + &
                cos(inpar%alpha)*E1perp/sqrt(sum(E1perp**2.0d0)) )
        end if
     end if

! Take half a step back to be ready for leapfrog action
     if (inpar%includephotonpressure) then
        call DustyBoris(velocity(:,1), velocity(:,1), Eperp+E1, B0, &
             -dt/2.0d0, inpar%mass, inpar%charge,inpar%Flight)
     else
        call Boris(velocity(:,1),velocity(:,1),Eperp+E1,B0,-dt/2.0d0, &
             inpar%mass,inpar%charge)
     end if

! calculate  vv, mu and pangle, for the first time step for saving later.
     call SpeedMomentAngle(vv(1),mu(1),pangle(1),velocity(:,1),B0,inpar%mass)

! Compute the trajectory - start of inner loop
     do jj=1,inpar%Nsteps-1

! Find fields through interpolation
        call FindFields(inpar%Ngridlevels, grid, &
             position(1,jj), position(2,jj), position(3,jj), E0, B0)

! Compute perpendicular E, i.e. remove artificial parallel component
        call PerpendicularEfield(E0, B0, Eperp)

! Find fluctuation E field if we are including such
        if (inpar%Efluctuations) then
           call FindFluct(inpar%rinfluct, inpar%routfluct, &
                inpar%E1x, inpar%E1y, inpar%E1z, inpar%ffluct, &
                position(1,jj), position(2,jj), position(3,jj), &
                fluctphase, dble(jj*dt), E1)
           E1mag = sqrt(sum(E1**2.0d0))
           if ( (inpar%usealpha) .and. (E1mag > 0.0d0) ) then
              call PerpendicularEfield(E1, B0, E1perp)
              E1par = E1 - E1perp
              E1 = E1mag * ( &
                   sin(inpar%alpha)*E1par/sqrt(sum(E1par**2.0d0)) + &
                   cos(inpar%alpha)*E1perp/sqrt(sum(E1perp**2.0d0)) )
           end if
        end if

! Boris gives us the new velocity
        if (inpar%includephotonpressure) then
           call DustyBoris(velocity(:,jj), velocity(:,jj+1), Eperp + E1, B0, &
                dt, inpar%mass, inpar%charge,inpar%Flight)
        else
           call Boris(velocity(:,jj), velocity(:,jj+1), Eperp + E1, B0, dt, &
                inpar%mass, inpar%charge)
        end if

! Now, push!
        position(:,jj+1) = position(:,jj) + velocity(:,jj+1)*dt

! Calculate step length
        step_since_saving = step_since_saving + position(:,jj+1) &
             - position(:,jj)
        if (sqrt(step_since_saving(1)**2.0d0 + step_since_saving(2)**2.0d0 + &
             step_since_saving(3)**2.0d0) .GE. inpar%saving_step_length) then
           saving_position(:,sjj) = position(:,jj+1)
           sjj = sjj + 1
           step_since_saving = 0.0d0
        end if

! Compute vv, mu and pangle, i.e. the speed magnetic moment and 
! pitch angle, so that these can be saved later. 
! This approximation uses the B-field at time step jj and the 
! velocity at jj+1, which is half a time step later. However, 
! as long as there is no E-field the result is exactly the same.

        call SpeedMomentAngle(vv(jj+1),mu(jj+1),pangle(jj+1), &
             velocity(:,jj+1),B0,inpar%mass)

! Check that we are still there
        rr=sqrt(sum(position(:,jj+1)**2))
        if ( rr<inpar%robst .or. &
             position(1,jj+1)<grid(1)%xmin .or. &
             position(1,jj+1)>grid(1)%xmax .or. &
             position(2,jj+1)<grid(1)%ymin .or. &
             position(2,jj+1)>grid(1)%ymax .or. &
             position(3,jj+1)<grid(1)%zmin .or. &
             position(3,jj+1)>grid(1)%zmax ) then
           exit
        end if

     end do


! Save the trajectory
     write (2,*) '% trajectory ', ii
     do kk=1,sjj-1


! Save only positions
!!$        write (2,fmt='(3E14.6)') &
!!$             saving_position(1), saving_position(2), saving_position(3)

! Save positions, speed, magnetic moment and pitch angle
        write (2,fmt='(6E14.6)') saving_position(1,kk), &
             saving_position(2,kk), saving_position(3,kk), &
             vv(kk), mu(kk), pangle(kk)
     end do
  end do
  
! Close the output file
  close(2)

end program cheese

