% cheese_a3pss15finalspeed computes, and makes plots of, the final speed
% for dust grains. The finals speed is the speed a dust grain has when it
% leaves the system.
%
% HG 2015-06-12


% The order of the experiments in the vectors is pss13, pss15-25

R=1e-9*[10 1 2 3 1 2 3 10 1 2 3 10].'*[1 1 1];
Q=[65 7 13 19 4 7 11 37 9 19 28 93]*elc;
m=[1.0472e-20 1.0472e-23 8.3776e-23 2.8274e-22 ...
   1.0472e-23 8.3776e-23 2.8274e-22 1.0472e-20 ...
   1.0472e-23 8.3776e-23 2.8274e-22 1.0472e-20];
Qm=Q./m;
vf=[9.6877 98.3444 47.6107 31.4267 74.4973 35.0203 23.9609 7.363 ...
    111.3319 57.4992 38.1079 11.5433]*1e3;

P  = polyfit(sqrt(Qm),vf,1);
PP = polyfit(1./R(1:4),vf(1:4),1);
PPg = polyfit(1./R(5:8),vf(5:8),1);
PPb = polyfit(1./R(9:12),vf(9:12),1);


figure(8004)
clf
set(gcf,'paperpositionmode','auto','position',[298 38 560 933])
ha(1)=axes('position',[0.12 0.73 0.85 0.25]);
plot(sqrt(Qm(5:8)),vf(5:8)/1000,'ko', ...
     sqrt(Qm(1:4)),vf(1:4)/1000,'kx', ...
     sqrt(Qm(9:12)),vf(9:12)/1000,'kd', ...
     [0 400],polyval(P,[0 400])/1000,'k', ...
     'linewidth',2,'markersize',10)
axis([0 400 0 1.2e2])
set(gca,'fontname','times','fontsize',14)
hhxl=xlabel('(Q/m)^{1/2}   [(As/kg)^{1/2}]','fontname','times','fontsize',18);
set(hhxl,'unit','normalized')
hhxlp=get(hhxl,'pos');
set(hhxl,'unit','normalized','pos',[hhxlp(1) -0.12 hhxlp(3)])

hhyl=ylabel('v_{f}   [km/s]','fontname','times','fontsize',18);
set(hhyl,'unit','normalized')
hhylp=get(hhyl,'pos');
set(hhyl,'unit','normalized','pos',[-0.05 hhylp(2:3)])


ha(2)=axes('position',[0.12 0.40 0.85 0.25]);
plot(1./R(5:8)*1e-8,vf(5:8)/1000,'ko', ...
     1./R(1:4)*1e-8,vf(1:4)/1000,'kx', ...
     1./R(9:12)*1e-8,vf(9:12)/1000,'kd', ...
     [0 1e9]*1e-8,polyval(PP,[0 1e9])/1000,'k', ...
     'linewidth',2,'markersize',10)
axis([0 10 0 1.2e2])
set(gca,'fontname','times','fontsize',14)
hhxl=xlabel('1/r_{d}   [10^{8}m^{-1}]','fontname','times','fontsize',18);
set(hhxl,'unit','normalized')
hhxlp=get(hhxl,'pos');
set(hhxl,'unit','normalized','pos',[hhxlp(1) -0.16 hhxlp(3)])

hhyl=ylabel('v_{f}   [km/s]','fontname','times','fontsize',18);
set(hhyl,'unit','normalized')
hhylp=get(hhyl,'pos');
set(hhyl,'unit','normalized','pos',[-0.05 hhylp(2:3)])

rr=10.^([-9.11982:0.05:-7]);

rho=2500; % density [kg/m^3]
veq=P(1)*sqrt((3*eps0/rho)*9.359769152789351)./rr;% equilibrium potential
veqplus4=P(1)*sqrt((3*eps0/rho)*13.359769152789351)./rr;
veqminus4=P(1)*sqrt((3*eps0/rho)*5.359769152789351)./rr;
veqplus8=P(1)*sqrt((3*eps0/rho)*17.359769152789351)./rr;
veqminus8=P(1)*sqrt((3*eps0/rho)*1.359769152789351)./rr;

indm8 = rr*5e9>=1.359769152789351; % index of fields below that critical
indm4 = rr*5e9>=5.359769152789351; % for Coulomb explosion
ind0  = rr*5e9>=9.359769152789351;
indp4 = rr*5e9>=13.359769152789351;
indp8 = rr*5e9>=17.359769152789351;

ha(3)=axes('position',[0.12 0.07 0.85 0.25]);
loglog(rr(indm8)*1e9,veqminus8(indm8)/1000,'k-.', ...
       rr(indm4)*1e9,veqminus4(indm4)/1000,'k.', ...
       rr(ind0)*1e9,veq(ind0)/1000,'k', ...
       rr(indp4)*1e9,veqplus4(indp4)/1000,'k:', ...
       rr(indp8)*1e9,veqplus8(indp8)/1000,'k--', ...
       'linewidth',2)
hold on
loglog(rr*1e9,10*ones(size(rr)),'k--',[1 1]*10,[0.5 150],'k--')
hold off
axis([1 100 1 150])
set(gca,'fontname','times','fontsize',14)
xlabel('r_{d}   [nm]','fontname','times','fontsize',18);
set(gca,'xtick',[1 2 3 5 10 20 30 50 100], ...
        'ytick',[1 2 3 5 10 20 30 50 100 150 200])
hold on
loglog(rr*1e9,10*ones(size(rr)),'k--',[1 1]*10,[0.5 150],'k--')
hold off
hhxl=xlabel('r_{d}   [nm]','fontname','times','fontsize',18);
set(hhxl,'unit','normalized')
hhxlp=get(hhxl,'pos');
set(hhxl,'unit','normalized','pos',[hhxlp(1) -0.15 hhxlp(3)])

hhyl=ylabel('v_{f}   [km/s]','fontname','times','fontsize',18);
set(hhyl,'unit','normalized')
hhylp=get(hhyl,'pos');
set(hhyl,'unit','normalized','pos',[-0.05 hhylp(2:3)])

for ii=1:3
  aa=axis(ha(ii));
  if strcmp(get(ha(ii),'yscale'),'log')
    text(aa(1)*exp(0.05*log(diff(aa(1:2)))), ...
         aa(3)*exp(0.93*log(diff(aa(3:4)))), ...
         ['(' char(96+ii) ')'],'parent',ha(ii), ...
         'fontname','times','fontsize',18) 
  else
    text(aa(1)+0.05*diff(aa(1:2)),aa(3)+0.93*diff(aa(3:4)), ...
         ['(' char(96+ii) ')'],'parent',ha(ii),'fontname','times','fontsize',18)
  end
end

hhl1=legend(ha(1),'V=5.4V','V=9.4V','V=13.4V');
set(hhl1,'pos',[0.7325 0.7315 0.217 0.0935])
hhl2=legend(ha(2),'V=5.4V','V=9.4V','V=13.4V');
set(hhl2,'pos',[0.7325 0.4175 0.217 0.0935])
hhl3=legend(ha(3),'V=1.4V','V=5.4V','V=9.4V','V=13.4V','V=17.4V');
set(hhl3,'pos',[0.7325 0.20 0.217 0.0935])

if 3==4
  ccc=pwd;
  cd ~
  print -f8004 -deps2 fig-dust-speed-revised.eps
  cd(ccc)
end
