% cheese_a3pss15ions plots outputs of statistical data from test particle
% simulations. This is a particular figure designed for publication in
% Planetary and Space Science. 
%
% HG 2015-06-04


figure(50000);clf
set(gcf,'paperpositionmode','auto','renderer','zbuffer')
set(gcf,'position',[6 331 1000 640])

c2=[];
for nn=1:2
  c1=jet(16*4^nn);
  c2=[c2;c1((1+(nn-1)*2*4^nn):nn*2*4^nn,:)];
end
c2=[c2;c1(nn*2*4^nn+1:end,:)];

c3=[];
for nn=1:3
  c1=jet(16*4^nn);
  c3=[c3;c1((1+(nn-1)*2*4^nn):nn*2*4^nn,:)];
end
c3=[c3;c1(nn*2*4^nn+1:end,:)];

c4=[];
for nn=1:4
  c1=jet(16*4^nn);
  c4=[c4;c1((1+(nn-1)*2*4^nn):nn*2*4^nn,:)];
end
c4=[c4;c1(nn*2*4^nn+1:end,:)];

% create axes
ha=zeros(1,8);
for jj=1:2
  for ii=1:3
    ha(1+ii+4*(jj-1))=axes('position', ...
                           [0.48+(jj-1)*0.28 (0.73-(ii-1)*0.32) 0.22 0.23]);
    set(ha(1+ii+4*(jj-1)),'nextplot','add')
  end
end

% line attributes
lw=[3 2 2 1];  % line width
colour='krgb';  % colour
style=['- ';'- ';'--'];

for kk=0:2
  for jj=1:2
    thedirectory = ['~/cheese/a3/pss15/collection/pss0' num2str(jj+2*kk)];
    cd(thedirectory)
    fid=fopen('inputcheese.m');
    theline=fgetl(fid);
    if length(theline)<5,
    theline=[theline '     '];
    end
    while ~(strcmp(theline(1:5),'%PARA') | strcmp(theline(1:5),'%para'))
      theline=fgetl(fid);
      if length(theline)<5,
        theline=[theline '     '];
      end
    end
    while ~(strcmp(theline(1:4),'%END') | strcmp(theline(1:4),'%end'))
      eval(theline)
      theline=fgetl(fid);
      if length(theline)<4,
        theline=[theline '    '];
      end
    end
    Lx=xmax-xmin;
    Ly=ymax-ymin;
    Lz=zmax-zmin;
    dx = Lx/(Nx-1);
    dy = Ly/(Ny-1);
    dz = Lz/(Nz-1);
    x = xmin + dx*[0:Nx-1];
    y = ymin + dy*[0:Ny-1];
    z = zmin + dz*[0:Nz-1];
    fclose(fid);

    y1=mean([zmin zmax]);

    dd=dir('outp/testpdens*.mat');
    Np=str2num(dd(end).name(10:19));

    % Assemble filenames
    nstring = ['outp/testpdens',num2str(Np,'%0.10d') '.cheese.mat'];


    % Load fields
    load(nstring)

    Nmidy=floor((y1-ymin)/dy+1);
    fract = (y1-ymin)/dy+1 - Nmidy;

    if Nprobes>=1
      dd=dir('outp/fp*.mat');
      load(['outp/' dd(end).name])
      Np=str2num(dd(end).name(3:12));
    end

    if kk==0
      ha(1+4*(jj-1))=axes('position',[0.05 0.56-(jj-1)*0.48 0.24 0.41]);
      pp=[];
      pp(:,:)=0.5*(n(:,Nmidy,:)*(1-fract)+n(:,Nmidy+1,:)*fract)/Np;pp=pp.';
      nmin=min(min(min(pp)));
      nmax=max(max(max(pp)));
      pp(isnan(pp))=nmax+1e16;
      ss=size(pp);
      pp=[pp zeros(ss(1),1);zeros(1,ss(2)+1)];
      xplot=[xmin-0.5*dx (x+0.5*dx)]*1e-3;
      zplot=[zmin-0.5*dz (z+0.5*dz)]*1e-3;
      surf(xplot,zplot,pp)
      view(2)
      set(gca,'fontname','times','fontsize',14)
      shading flat
      view(2)
      axis equal
      axis([min(xplot) max(xplot) min(zplot) max(zplot)])
      hh=colorbar;
      hhpos=get(hh,'position');
      set(hh,'position',[hhpos(1)+0.06 0.58-(jj-1)*0.48 0.01 hhpos(4)*1.25])
      set(hh,'fontname','times','fontsize',16)
      ylabel(hh,'n/Q   [m^{-3}s]','fontname','times','fontsize',18)
      hhylpos=get(get(hh,'ylabel'),'pos');
      set(get(hh,'ylabel'),'position',[hhylpos(1)*1.25 hhylpos(2:3)])
      xlabel('x   [km]','fontname','times','fontsize',18)
      ylabel('z   [km]','fontname','times','fontsize',18)
      hhylpos=get(get(ha(1+4*(jj-1)),'ylabel'),'pos');
      set(get(ha(1+4*(jj-1)),'ylabel'), ...
          'position',[hhylpos(1)*0.95 hhylpos(2:3)])
      colormap(c3)

      hold on
      for ii=1:Nprobes
        % Probe position
        z43 = probe(ii).xc+1i*probe(ii).zc + ...
              probe(ii).rprobe*exp(1i*[0:360]*pi/180);
        plot3(ha(1+4*(jj-1)), ...
              real(z43)*1e-3,imag(z43)*1e-3,ones(size(z43))*nmax*2,'r', ...
              'linewidth',2)
        % E-field arrow
        pil(2.5e4*(1+1i),1i,5e3,'w');
        plot([2.5e4*(1+1i) 2.5e4+1.0e4*1i],'w','linewidth',2)
        text(2.0e4,2.5e4,'E','color','w','fontname','times','fontsize',18)
        % Source size label
        text(0.0e4,2.3e4,['r_{s}=' num2str(rsource*1e-3) ' km'], ...
             'color','w','horizontalalignment','center', ...
             'fontname','times','fontsize',18)
        % B-field cross and ring
        zBc=2.5e4*(1-1i); rB=2.5e3;
        zB=rB*exp(1i*[0:360]*pi/180) + zBc;
        plot(zB,'w','linewidth',2)
        plot(zBc+rB*[(-1+1i) (1-1i)]/sqrt(2),'w','linewidth',2)
        plot(zBc+rB*[(-1-1i) (1+1i)]/sqrt(2),'w','linewidth',2)
        text(2.0e4,-2.1e4,'B','color','w','fontname','times','fontsize',18)
     end
      hold off
    end



    fxy=[]; fyz=[]; fx=[]; fy=[]; fz=[];
    fxy=(0.5*probe(1).f(:,:,1) + sum(probe(1).f(:,:,2:end-1),3) + ...
         0.5*probe(1).f(:,:,end))*probe(1).dvz/Np; 
    fx = (0.5*fxy(:,1) + sum(fxy(:,2:end-1),2) + 0.5*fxy(:,end))*probe(1).dvy;
    fy = (0.5*fxy(1,:) + sum(fxy(2:end-1,:),1) + 0.5*fxy(end,:))*probe(1).dvx;
    fyz(:,:)=(0.5*probe(1).f(1,:,:) + sum(probe(1).f(2:end-1,:,:),1) + ...
              0.5*probe(1).f(end,:,:))*probe(1).dvx/Np;
    fz = (0.5*fyz(1,:) + sum(fyz(2:end-1,:),1) + 0.5*fyz(end,:))*probe(1).dvy;

    Ix = 0.5*sum(fz(1:end-1)+fz(2:end))*probe(1).dvz*1e-3; % we use km/s
    Iy = 0.5*sum(fz(1:end-1)+fz(2:end))*probe(1).dvz*1e-3; % instead of m/s
    Iz = 0.5*sum(fz(1:end-1)+fz(2:end))*probe(1).dvz*1e-3; % in the figures

    plot(ha(2+4*(jj-1)),probe(1).vx*1e-3,fx/Ix,[colour(kk+1) style(kk+1,:)], ...
         'linewidth',lw(kk+1))
    plot(ha(3+4*(jj-1)),probe(1).vy*1e-3,fy/Iy,[colour(kk+1) style(kk+1,:)], ...
         'linewidth',lw(kk+1))
    plot(ha(4+4*(jj-1)),probe(1).vz*1e-3,fz/Iz,[colour(kk+1) style(kk+1,:)], ...
         'linewidth',lw(kk+1))
    xlabel(ha(2+4*(jj-1)),'v_{x}   [km/s]','fontname','times','fontsize',18)
    xlabel(ha(3+4*(jj-1)),'v_{y}   [km/s]','fontname','times','fontsize',18)
    xlabel(ha(4+4*(jj-1)),'v_{z}   [km/s]','fontname','times','fontsize',18)
    set(ha(2:4+4*(jj-1)),'fontname','times','fontsize',14)
  end
end
ylabel(ha(2),'f(v_{x})   [(km/s)^{-1}]','fontname','times','fontsize',18)
ylabel(ha(3),'f(v_{y})   [(km/s)^{-1}]','fontname','times','fontsize',18)
ylabel(ha(4),'f(v_{z})   [(km/s)^{-1}]','fontname','times','fontsize',18)


axis(ha(2),[-160 -60 0 0.09])
axis(ha(3),[-12 12 0 0.55])
axis(ha(4),[0 110 0 0.08])
axis(ha(6),[-140 -60 0 0.05])
axis(ha(7),[-20 20 0 0.11])
axis(ha(8),[0 110 0 0.06])

colour='wkkkwkkkwkkkwkkkwkkkwkkkwkkkwkkk';
for ii=1:length(ha)
  aa=axis(ha(ii));
  vposn=0.5 - 0.35*(double(colour(ii))-113)/6;
  text(aa(1)+0.05*diff(aa(1:2)),aa(3)+vposn*diff(aa(3:4)), ...
       ['(' char(96+ii) ')'],'parent',ha(ii), ...
       'color',colour(ii),'fontname','times','fontsize',18)
end

hht=title(ha(2),'r_{s}=150 km','fontname','times','fontsize',18);
hhtpos=get(hht,'position');
set(hht,'position',[hhtpos(1) hhtpos(2)*0.88 hhtpos(3)])

hht=title(ha(6),'r_{s}=750 km','fontname','times','fontsize',18);
hhtpos=get(hht,'position');
set(hht,'position',[hhtpos(1) hhtpos(2)*0.88 hhtpos(3)])

if 3==4
  ccc=pwd;
  cd ~
  print -f50000 -depsc2 fig-water-ions-revised.eps
  cd(ccc)
end
